<?php

namespace Minimog\Woo;

defined( 'ABSPATH' ) || exit;

class Fly_Cart {
	protected static $instance = null;

	public static function instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	public function initialize() {
		add_action( 'wp_footer', [ $this, 'fly_cart_template' ] );

		/**
		 * We need calculate cart total value when removed a coupon.
		 * Priority 1 required to calculate before value set to session
		 *
		 * @hooked woocommerce_removed_coupon
		 * @see    \WC_Cart_Session::set_session()
		 */
		add_action( 'woocommerce_removed_coupon', [ $this, 'update_cart_total' ], 1 );

		/**
		 * Change button order.
		 */
		remove_action( 'woocommerce_widget_shopping_cart_buttons', 'woocommerce_widget_shopping_cart_button_view_cart', 10 );
		remove_action( 'woocommerce_widget_shopping_cart_buttons', 'woocommerce_widget_shopping_cart_proceed_to_checkout', 20 );

		add_action( 'woocommerce_widget_shopping_cart_buttons', 'woocommerce_widget_shopping_cart_proceed_to_checkout', 10 );
		add_action( 'woocommerce_widget_shopping_cart_buttons', [ $this, 'shopping_cart_button_view_cart' ], 20 );

		add_action( 'wp_ajax_minimog_update_product_quantity', [ $this, 'update_product_quantity' ] );
		add_action( 'wp_ajax_nopriv_minimog_update_product_quantity', [ $this, 'update_product_quantity' ] );

		add_filter( 'woocommerce_checkout_get_value', [ $this, 'output_order_notes' ], 10, 2 );
		add_action( 'wp_ajax_minimog_save_order_notes', [ $this, 'save_order_notes' ] );
		add_action( 'wp_ajax_nopriv_minimog_save_order_notes', [ $this, 'save_order_notes' ] );

		add_action( 'wp_ajax_minimog_calculate_shipping', [ $this, 'calculate_shipping' ] );
		add_action( 'wp_ajax_nopriv_minimog_calculate_shipping', [ $this, 'calculate_shipping' ] );

		add_action( 'wp_ajax_minimog_apply_coupon', [ $this, 'apply_coupon' ] );
		add_action( 'wp_ajax_nopriv_minimog_apply_coupon', [ $this, 'apply_coupon' ] );

		add_action( 'wp_ajax_minimog_remove_coupon', [ $this, 'remove_coupon' ] );
		add_action( 'wp_ajax_nopriv_minimog_remove_coupon', [ $this, 'remove_coupon' ] );
	}

	/**
	 * @see \WC_Shortcode_Cart::output()
	 */
	function calculate_shipping() {
		// Constants.
		wc_maybe_define_constant( 'WOOCOMMERCE_CART', true );

		\WC_Shortcode_Cart::calculate_shipping();

		WC()->cart->calculate_totals();

		$fragments = array();

		ob_start();
		wc_get_template( 'cart/cart-totals-table.php' );
		$fragments['.cart-totals-table'] = ob_get_clean();

		ob_start();
		wc_print_notices();
		$notices_html = ob_get_clean();

		ob_start();
		echo '<div class="woocommerce-notices-wrapper">' . $notices_html . '</div>';
		$fragments['.woocommerce-notices-wrapper'] = ob_get_clean();

		ob_start();
		echo '<div class="fly-cart-messages">' . $notices_html . '</div>';
		$fragments['.fly-cart-messages'] = ob_get_clean();

		wp_send_json_success( [
			'fragments' => $fragments,
		] );
	}

	public function fly_cart_template() {
		if ( ! \Minimog_Global::instance()->get_fly_cart() ) {
			return;
		}

		wc_get_template( 'cart/fly-cart.php' );
	}

	public function shopping_cart_button_view_cart() {
		\Minimog_Templates::render_button( [
			'text'        => esc_html__( 'View cart', 'minimog' ),
			'link'        => [
				'url' => esc_url( wc_get_cart_url() ),
			],
			'extra_class' => 'view-cart',
			'style'       => 'bottom-line',
		] );
	}

	public function update_product_quantity() {
		if ( ! empty( $_POST['cart_item_key'] ) && isset( $_POST['cart_item_qty'] ) ) {
			if ( WC()->cart->get_cart_item( $_POST['cart_item_key'] ) ) {
				if ( (float) $_POST['cart_item_qty'] > 0 ) {
					WC()->cart->set_quantity( $_POST['cart_item_key'], (float) $_POST['cart_item_qty'] );
				} else {
					WC()->cart->remove_cart_item( $_POST['cart_item_key'] );
				}

				WC()->cart->check_cart_coupons();

				wp_send_json_success();
			}

			wp_send_json_error( [
				'message' => esc_html__( 'Cart key not exist!', 'minimog' ),
			] );
		}

		wp_send_json_error();
	}

	public function update_cart_total( $coupon ) {
		WC()->cart->calculate_totals();
	}

	public function save_order_notes() {
		$order_notes = isset( $_POST['order_notes'] ) ? sanitize_textarea_field( $_POST['order_notes'] ) : '';

		WC()->session->set( 'minimog_order_notes', $order_notes );

		wc_add_notice( __( 'Your order notes saved.', 'minimog' ) );

		$fragments = array();

		ob_start();
		wc_print_notices();
		$notices_html = ob_get_clean();

		ob_start();
		echo '<div class="woocommerce-notices-wrapper">' . $notices_html . '</div>';
		$fragments['.woocommerce-notices-wrapper'] = ob_get_clean();

		ob_start();
		echo '<div class="fly-cart-messages">' . $notices_html . '</div>';
		$fragments['.fly-cart-messages'] = ob_get_clean();

		wp_send_json_success( [
			'fragments' => $fragments,
		] );
	}

	public function output_order_notes( $value, $input ) {
		if ( 'order_comments' === $input ) {
			$notes = WC()->session->get( 'minimog_order_notes' );

			if ( ! empty( $notes ) ) {
				return $notes;
			}
		}

		return $value;
	}

	/**
	 * AJAX apply coupon.
	 *
	 * @see \WC_AJAX::apply_coupon()
	 */
	public function apply_coupon() {
		check_ajax_referer( 'apply-coupon', 'security' );

		$fragments = array();

		if ( ! empty( $_POST['coupon_code'] ) ) {
			\WC()->cart->add_discount( wc_format_coupon_code( wp_unslash( $_POST['coupon_code'] ) ) ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized

			ob_start();
			wc_get_template( 'cart/cart-totals-table.php' );
			$fragments['.cart-totals-table'] = ob_get_clean();

			$this->get_cart_goal_text_fragment( $fragments );
		} else {
			wc_add_notice( \WC_Coupon::get_generic_coupon_error( \WC_Coupon::E_WC_COUPON_PLEASE_ENTER ), 'error' );
		}

		ob_start();
		wc_print_notices();
		$notices_html = ob_get_clean();

		ob_start();
		echo '<div class="woocommerce-notices-wrapper">' . $notices_html . '</div>';
		$fragments['.woocommerce-notices-wrapper'] = ob_get_clean();

		ob_start();
		echo '<div class="fly-cart-messages">' . $notices_html . '</div>';
		$fragments['.fly-cart-messages'] = ob_get_clean();

		wp_send_json_success( [
			'fragments' => $fragments,
		] );
	}

	/**
	 * AJAX remove coupon.
	 *
	 * @see \WC_AJAX::remove_coupon()
	 */
	public function remove_coupon() {
		check_ajax_referer( 'remove-coupon', 'security' );

		$coupon    = isset( $_POST['coupon_code'] ) ? wc_format_coupon_code( wp_unslash( $_POST['coupon_code'] ) ) : false; // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		$fragments = array();

		if ( empty( $coupon ) ) {
			wc_add_notice( __( 'Sorry there was a problem removing this coupon.', 'minimog' ), 'error' );
		} else {
			WC()->cart->remove_coupon( $coupon );
			wc_add_notice( __( 'Coupon has been removed.', 'minimog' ) );

			ob_start();
			wc_get_template( 'cart/cart-totals-table.php' );
			$fragments['.cart-totals-table'] = ob_get_clean();

			$this->get_cart_goal_text_fragment( $fragments );
		}

		ob_start();
		wc_print_notices();
		$notices_html = ob_get_clean();

		ob_start();
		echo '<div class="woocommerce-notices-wrapper">' . $notices_html . '</div>';
		$fragments['.woocommerce-notices-wrapper'] = ob_get_clean();

		ob_start();
		echo '<div class="fly-cart-messages">' . $notices_html . '</div>';
		$fragments['.fly-cart-messages'] = ob_get_clean();

		wp_send_json_success( [ 'fragments' => $fragments ] );
	}

	public function get_cart_goal_text_fragment( &$fragments ) {
		$amount_for_free_shipping = Free_Shipping_Label::instance()->get_min_free_shipping_amount();
		$min_amount               = (float) $amount_for_free_shipping['amount'];

		if ( $min_amount > 0 ) {
			ob_start();
			wc_get_template( 'cart/cart-goal-text.php', [
				'min_amount' => $min_amount,
			] );
			$fragments['.cart-goal-text'] = ob_get_clean();
		}

		return $fragments;
	}
}

Fly_Cart::instance()->initialize();
