<?php

namespace Minimog\Woo;

defined( 'ABSPATH' ) || exit;

class Product_Query {
	protected static $instance = null;

	/**
	 * Stores chosen attributes.
	 *
	 * @var array
	 */
	private static $chosen_attributes;

	public static function instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	public function initialize() {
		add_action( 'wp_ajax_minimog_get_products', [ $this, 'get_products' ] );
		add_action( 'wp_ajax_nopriv_minimog_get_products', [
			$this,
			'get_products',
		] );

		/**
		 * Change main query for shop catalog.
		 */
		add_filter( 'loop_shop_per_page', [ $this, 'loop_shop_per_page' ], 20 );

		add_filter( 'loop_shop_post_in', [ $this, 'filter_by_ids' ] ); // For Url Filter.
		add_filter( 'minimog/product_query/post_in', [ $this, 'filter_by_ids' ] ); // For Ajax Filter.

		add_filter( 'woocommerce_product_query_tax_query', [
			$this,
			'update_product_query_tax_query',
		], 10, 2 ); // For Url Filter.
		add_filter( 'minimog/product_query/tax_query', [
			$this,
			'update_product_query_tax_query',
		], 10, 2 ); // For Ajax Filter.

		/**
		 * Add query vars min_price + max_price
		 */
		add_action( 'woocommerce_product_query', [ $this, 'add_wc_query_vars' ], 10, 2 );
	}

	public function add_wc_query_vars( $q, $wc_query ) {
		if ( isset( $_GET['min_price'] ) && isset( $_GET['max_price'] ) ) {
			$q->set( 'min_price', $_GET['min_price'] );
			$q->set( 'max_price', $_GET['max_price'] );
		}
	}

	public function get_products() {
		$base_url = isset( $_GET['base_url'] ) ? sanitize_text_field( $_GET['base_url'] ) : '';
		$current  = isset( $_GET['product-page'] ) ? intval( sanitize_text_field( $_GET['product-page'] ) ) : 1;
		$per_page = isset( $_GET['posts_per_page'] ) ? intval( sanitize_text_field( $_GET['posts_per_page'] ) ) : \Minimog::setting( 'shop_archive_number_item' );

		$showingWidgets = isset( $_GET['widgets'] ) ? $_GET['widgets'] : array();

		$orig_request_uri = $_SERVER['REQUEST_URI'];

		// Overwrite the REQUEST_URI variable.
		$_SERVER['REQUEST_URI'] = $base_url;

		global $wp_query;
		$clone_wp_query = $wp_query;

		$query_args = [
			'post_type'      => 'product',
			'post_status'    => 'publish',
			'posts_per_page' => $per_page,
			'paged'          => $current,
		];

		$ordering              = $this->get_catalog_ordering_args();
		$query_args['orderby'] = $ordering['orderby'];
		$query_args['order']   = $ordering['order'];

		if ( isset( $ordering['meta_key'] ) ) {
			$query_args['meta_key'] = $ordering['meta_key'];
		}

		$post_in = array_unique( (array) apply_filters( 'minimog/product_query/post_in', array() ) );
		if ( ! empty( $post_in ) ) {
			$query_args['post__in'] = $post_in;
		}

		$tax_query = $this->get_tax_query();

		if ( ! empty( $_GET['is_tax'] ) ) {
			$current_tax_name = $_GET['is_current_tax'];
			$current_term_id  = $_GET['is_current_term_id'];

			if ( ! empty( $current_tax_name ) && ! empty( $current_term_id ) ) {
				$tax_query[] = [
					'taxonomy' => $current_tax_name,
					'field'    => 'term_id',
					'terms'    => [ $current_term_id ],
					'operator' => 'IN',
				];
			}
		}

		if ( ! empty( $tax_query ) ) {
			$query_args['tax_query'] = $tax_query;
		}

		$meta_query = self::get_main_meta_query();

		if ( ! empty( $meta_query ) ) {
			$query_args['meta_query'] = $meta_query;
		}

		// Additional hooks to change WP Query.
		add_filter( 'posts_clauses', [ $this, 'product_query_post_clauses' ], 10, 2 );

		$query = new \WP_Query( $query_args );

		$query->set( 'wc_query', 'product_query' );

		if ( ( isset( $_GET['max_price'] ) && isset( $_GET['min_price'] ) ) ) {
			$query->set( 'min_price', $_GET['min_price'] );
			$query->set( 'max_price', $_GET['max_price'] );
		}

		$wp_query = $query;

		$total = $query->found_posts;

		$result_count_html = \Minimog_Woo::instance()->get_result_count_text( $current, $per_page, $total );

		$fragments = [];

		if ( ! empty( $showingWidgets ) ) {
			foreach ( $showingWidgets as $widget ) {
				$type     = $widget['name'];
				$instance = $widget['instance'];
				// Get only content html.
				$instance['widget_content_only'] = true;

				global $wp_widget_factory;

				// to avoid unwanted warnings let's check before using widget.
				if ( is_object( $wp_widget_factory ) && isset( $wp_widget_factory->widgets, $wp_widget_factory->widgets[ $type ] ) ) {
					ob_start();
					the_widget( $type, $instance );
					$widget_new_html = ob_get_clean();

					$fragments[ $widget['id'] . ' .widget-content' ] = $widget_new_html;
				}
			}
		}

		if ( $query->have_posts() ) :
			wc_set_loop_prop( 'loop', 0 );

			ob_start();

			echo '<div class="grid-sizer"></div>';

			while ( $query->have_posts() ) : $query->the_post();
				/**
				 * Hook: woocommerce_shop_loop.
				 *
				 * @hooked WC_Structured_Data::generate_product_data() - 10
				 */
				do_action( 'woocommerce_shop_loop' );

				minimog_get_wc_template_part( 'content-product', '', [
					'settings' => [
						'show_list_view' => true,
					],
				] );
				?>
			<?php endwhile; ?>
			<?php
			wp_reset_postdata();

			$template = ob_get_clean();
			$template = preg_replace( '~>\s+<~', '><', $template );
			$success  = true;
		else :
			$template = esc_html__( 'Sorry, we can not find any products for this search.', 'minimog' );
			$success  = false;
		endif;

		ob_start();
		\Minimog_Woo::instance()->get_pagination();
		$pagination = ob_get_clean();

		$active_filters = \Minimog_Woo::instance()->get_remove_active_filter_links( $_GET );

		$fragments['.archive-result-count p']                  = $result_count_html;
		$fragments['.woocommerce-pagination']                  = $pagination;
		$fragments['#active-filters-bar .active-filters-list'] = $active_filters;

		$response = [
			'success'   => $success,
			'template'  => $template,
			'fragments' => apply_filters( 'minimog/get_products/fragments', $fragments ),
		];

		$wp_query = $clone_wp_query;

		// Restore the original REQUEST_URI - in case anything else would resort on it.
		$_SERVER['REQUEST_URI'] = $orig_request_uri;

		wp_send_json( $response );
	}

	public function loop_shop_per_page() {
		$per_page = \Minimog::setting( 'shop_archive_number_item' );

		return isset( $_GET['product_per_page'] ) ? wc_clean( $_GET['product_per_page'] ) : $per_page;
	}

	public function filter_by_ids( $ids ) {
		if ( ! empty( $_GET['highlight_filter'] ) ) {
			$highlight_filter = wc_clean( $_GET['highlight_filter'] );

			switch ( $highlight_filter ) {
				case 'on_sale':
					$product_ids_on_sale = wc_get_product_ids_on_sale();
					$ids                 = array_merge( $ids, $product_ids_on_sale );

					break;
				case 'best_selling':
					$product_ids_best_selling = \Minimog_Woo::instance()->get_product_ids_best_selling();
					$ids                      = array_merge( $ids, $product_ids_best_selling );

					break;
				case 'new_arrivals':
					$product_ids_new_arrivals = \Minimog_Woo::instance()->get_product_ids_new_arrivals();
					$ids                      = array_merge( $ids, $product_ids_new_arrivals );

					break;
			}
		}

		return $ids;
	}

	/**
	 * @param array     $tax_query
	 * @param \WC_Query $wc_query
	 *
	 * @return mixed
	 */
	public function update_product_query_tax_query( $tax_query, $wc_query ) {
		// Filter by featured.
		if ( ! empty( $_GET['highlight_filter'] ) ) {
			$highlight_filter = wc_clean( $_GET['highlight_filter'] );

			switch ( $highlight_filter ) {
				case 'featured':
					$tax_query[] = array(
						'taxonomy'         => 'product_visibility',
						'terms'            => 'featured',
						'field'            => 'name',
						'operator'         => 'IN',
						'include_children' => false,
					);
					break;
			}
		}

		// Filter by category.
		if ( isset( $_GET['filter_product_cat'] ) ) { // WPCS: input var ok, CSRF ok.
			$cats = array_filter( array_map( 'absint', explode( ',', $_GET['filter_product_cat'] ) ) ); // WPCS: input var ok, CSRF ok, Sanitization ok.

			if ( $cats ) {
				$tax_query[] = array(
					'taxonomy' => 'product_cat',
					'terms'    => $cats,
					'operator' => 'IN',
				);
			}
		}

		// Filter by tag.
		if ( isset( $_GET['filter_product_tag'] ) ) { // WPCS: input var ok, CSRF ok.
			$tags = array_filter( array_map( 'absint', explode( ',', $_GET['filter_product_tag'] ) ) ); // WPCS: input var ok, CSRF ok, Sanitization ok.

			if ( $tags ) {
				$tax_query[] = array(
					'taxonomy' => 'product_tag',
					'terms'    => $tags,
					'operator' => 'IN',
				);
			}
		}

		if ( isset( $_GET['filter_product_brand'] ) ) { // WPCS: input var ok, CSRF ok.
			$brands = array_filter( array_map( 'absint', explode( ',', $_GET['filter_product_brand'] ) ) ); // WPCS: input var ok, CSRF ok, Sanitization ok.

			if ( $brands ) {
				$tax_query[] = array(
					'taxonomy' => 'product_brand',
					'terms'    => $brands,
					'operator' => 'IN',
				);
			}
		}

		return $tax_query;
	}

	/**
	 * Add extra clauses to the product query.
	 *
	 * @param array     $args     Product query clauses.
	 * @param \WP_Query $wp_query The current product query.
	 *
	 * @return array The updated product query clauses array.
	 */
	public function product_query_post_clauses( $args, $wp_query ) {
		$args = $this->price_filter_post_clauses( $args, $wp_query );

		//$args = $this->filterer->filter_by_attribute_post_clauses( $args, $wp_query, $this->get_layered_nav_chosen_attributes() );

		return $args;
	}

	public function get_catalog_ordering_args( $orderby = '', $order = '' ) {
		// Get ordering from query string unless defined.
		if ( ! $orderby ) {
			// phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$orderby_value = isset( $_GET['orderby'] ) ? wc_clean( (string) wp_unslash( $_GET['orderby'] ) ) : wc_clean( get_query_var( 'orderby' ) );

			if ( ! $orderby_value ) {
				if ( is_search() ) {
					$orderby_value = 'relevance';
				} else {
					$orderby_value = apply_filters( 'woocommerce_default_catalog_orderby', get_option( 'woocommerce_default_catalog_orderby', 'menu_order' ) );
				}
			}

			// Get order + orderby args from string.
			$orderby_value = is_array( $orderby_value ) ? $orderby_value : explode( '-', $orderby_value );
			$orderby       = esc_attr( $orderby_value[0] );
			$order         = ! empty( $orderby_value[1] ) ? $orderby_value[1] : $order;
		}

		// Convert to correct format.
		$orderby = strtolower( is_array( $orderby ) ? (string) current( $orderby ) : (string) $orderby );
		$order   = strtoupper( is_array( $order ) ? (string) current( $order ) : (string) $order );
		$args    = array(
			'orderby'  => $orderby,
			'order'    => ( 'DESC' === $order ) ? 'DESC' : 'ASC',
			'meta_key' => '', // @codingStandardsIgnoreLine
		);

		switch ( $orderby ) {
			case 'id':
				$args['orderby'] = 'ID';
				break;
			case 'menu_order':
				$args['orderby'] = 'menu_order title';
				break;
			case 'title':
				$args['orderby'] = 'title';
				$args['order']   = ( 'DESC' === $order ) ? 'DESC' : 'ASC';
				break;
			case 'relevance':
				$args['orderby'] = 'relevance';
				$args['order']   = 'DESC';
				break;
			case 'rand':
				$args['orderby'] = 'rand'; // @codingStandardsIgnoreLine
				break;
			case 'date':
				$args['orderby'] = 'date ID';
				$args['order']   = ( 'ASC' === $order ) ? 'ASC' : 'DESC';
				break;
			case 'price':
				$callback = 'DESC' === $order ? 'order_by_price_desc_post_clauses' : 'order_by_price_asc_post_clauses';
				add_filter( 'posts_clauses', array( $this, $callback ) );
				break;
			case 'popularity':
				add_filter( 'posts_clauses', array( $this, 'order_by_popularity_post_clauses' ) );
				break;
			case 'rating':
				add_filter( 'posts_clauses', array( $this, 'order_by_rating_post_clauses' ) );
				break;
		}

		return apply_filters( 'woocommerce_get_catalog_ordering_args', $args, $orderby, $order );
	}

	/**
	 * Custom query used to filter products by price.
	 *
	 * @since 3.6.0
	 *
	 * @param array     $args     Query args.
	 * @param \WP_Query $wp_query WP_Query object.
	 *
	 * @return array
	 */
	public function price_filter_post_clauses( $args, $wp_query ) {
		global $wpdb;

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( ( ! isset( $_GET['max_price'] ) && ! isset( $_GET['min_price'] ) ) ) {
			return $args;
		}

		// phpcs:disable WordPress.Security.NonceVerification.Recommended
		$current_min_price = isset( $_GET['min_price'] ) ? floatval( wp_unslash( $_GET['min_price'] ) ) : 0;
		$current_max_price = isset( $_GET['max_price'] ) ? floatval( wp_unslash( $_GET['max_price'] ) ) : PHP_INT_MAX;
		// phpcs:enable WordPress.Security.NonceVerification.Recommended

		/**
		 * Adjust if the store taxes are not displayed how they are stored.
		 * Kicks in when prices excluding tax are displayed including tax.
		 */
		if ( wc_tax_enabled() && 'incl' === get_option( 'woocommerce_tax_display_shop' ) && ! wc_prices_include_tax() ) {
			$tax_class = apply_filters( 'woocommerce_price_filter_widget_tax_class', '' ); // Uses standard tax class.
			$tax_rates = \WC_Tax::get_rates( $tax_class );

			if ( $tax_rates ) {
				$current_min_price -= \WC_Tax::get_tax_total( \WC_Tax::calc_inclusive_tax( $current_min_price, $tax_rates ) );
				$current_max_price -= \WC_Tax::get_tax_total( \WC_Tax::calc_inclusive_tax( $current_max_price, $tax_rates ) );
			}
		}

		$args['join']  = $this->append_product_sorting_table_join( $args['join'] );
		$args['where'] .= $wpdb->prepare(
			' AND NOT (%f<wc_product_meta_lookup.min_price OR %f>wc_product_meta_lookup.max_price ) ',
			$current_max_price,
			$current_min_price
		);

		return $args;
	}

	/**
	 * Handle numeric price sorting.
	 *
	 * @param array $args Query args.
	 *
	 * @return array
	 */
	public function order_by_price_asc_post_clauses( $args ) {
		$args['join']    = $this->append_product_sorting_table_join( $args['join'] );
		$args['orderby'] = ' wc_product_meta_lookup.min_price ASC, wc_product_meta_lookup.product_id ASC ';

		return $args;
	}

	/**
	 * Handle numeric price sorting.
	 *
	 * @param array $args Query args.
	 *
	 * @return array
	 */
	public function order_by_price_desc_post_clauses( $args ) {
		$args['join']    = $this->append_product_sorting_table_join( $args['join'] );
		$args['orderby'] = ' wc_product_meta_lookup.max_price DESC, wc_product_meta_lookup.product_id DESC ';

		return $args;
	}

	/**
	 * WP Core does not let us change the sort direction for individual orderby params - https://core.trac.wordpress.org/ticket/17065.
	 *
	 * This lets us sort by meta value desc, and have a second orderby param.
	 *
	 * @param array $args Query args.
	 *
	 * @return array
	 */
	public function order_by_popularity_post_clauses( $args ) {
		$args['join']    = $this->append_product_sorting_table_join( $args['join'] );
		$args['orderby'] = ' wc_product_meta_lookup.total_sales DESC, wc_product_meta_lookup.product_id DESC ';

		return $args;
	}

	/**
	 * Order by rating post clauses.
	 *
	 * @param array $args Query args.
	 *
	 * @return array
	 */
	public function order_by_rating_post_clauses( $args ) {
		$args['join']    = $this->append_product_sorting_table_join( $args['join'] );
		$args['orderby'] = ' wc_product_meta_lookup.average_rating DESC, wc_product_meta_lookup.rating_count DESC, wc_product_meta_lookup.product_id DESC ';

		return $args;
	}

	/**
	 * Join wc_product_meta_lookup to posts if not already joined.
	 *
	 * @param string $sql SQL join.
	 *
	 * @return string
	 */
	private function append_product_sorting_table_join( $sql ) {
		global $wpdb;

		if ( ! strstr( $sql, 'wc_product_meta_lookup' ) ) {
			$sql .= " LEFT JOIN {$wpdb->wc_product_meta_lookup} wc_product_meta_lookup ON $wpdb->posts.ID = wc_product_meta_lookup.product_id ";
		}

		return $sql;
	}

	/**
	 * Pre_get_posts above may adjust the main query to add WooCommerce logic. When this query is done, we need to ensure
	 * all custom filters are removed.
	 *
	 * This is done here during the_posts filter. The input is not changed.
	 *
	 * @param array $posts Posts from WP Query.
	 *
	 * @return array
	 */
	public function remove_product_query_filters( $posts ) {
		$this->remove_ordering_args();
		remove_filter( 'posts_clauses', array( $this, 'price_filter_post_clauses' ), 10 );

		return $posts;
	}

	/**
	 * Remove ordering queries.
	 */
	public function remove_ordering_args() {
		remove_filter( 'posts_clauses', array( $this, 'order_by_price_asc_post_clauses' ) );
		remove_filter( 'posts_clauses', array( $this, 'order_by_price_desc_post_clauses' ) );
		remove_filter( 'posts_clauses', array( $this, 'order_by_popularity_post_clauses' ) );
		remove_filter( 'posts_clauses', array( $this, 'order_by_rating_post_clauses' ) );
	}

	public static function get_main_tax_query() {
		global $wp_query;

		$tax_query = isset( $wp_query->tax_query, $wp_query->tax_query->queries ) ? $wp_query->tax_query->queries : array();

		return $tax_query;
	}

	/**
	 * Get the meta query which was used by the main query.
	 *
	 * @return array
	 */
	public static function get_main_meta_query() {
		global $wp_query;

		$args       = $wp_query->query_vars;
		$meta_query = isset( $args['meta_query'] ) ? $args['meta_query'] : array();

		return $meta_query;
	}

	public static function is_main_query_has_post() {
		global $wp_query;

		return isset( $wp_query->post_count ) && $wp_query->post_count > 0 ? true : false;
	}

	/**
	 * Based on WP_Query::parse_search
	 */
	public static function get_main_search_query_sql() {
		global $wpdb;
		global $wp_query;

		$args = $wp_query->query_vars;

		$search_terms = isset( $args['search_terms'] ) ? $args['search_terms'] : array();
		$sql          = array();

		foreach ( $search_terms as $term ) {
			// Terms prefixed with '-' should be excluded.
			$include = '-' !== substr( $term, 0, 1 );

			if ( $include ) {
				$like_op  = 'LIKE';
				$andor_op = 'OR';
			} else {
				$like_op  = 'NOT LIKE';
				$andor_op = 'AND';
				$term     = substr( $term, 1 );
			}

			$like = '%' . $wpdb->esc_like( $term ) . '%';
			// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
			$sql[] = $wpdb->prepare( "(($wpdb->posts.post_title $like_op %s) $andor_op ($wpdb->posts.post_excerpt $like_op %s) $andor_op ($wpdb->posts.post_content $like_op %s))", $like, $like, $like );
		}

		if ( ! empty( $sql ) && ! is_user_logged_in() ) {
			$sql[] = "($wpdb->posts.post_password = '')";
		}

		return implode( ' AND ', $sql );
	}

	public function get_main_price_query_sql( $min_price = null, $max_price = null ) {
		global $wp_query;
		global $wpdb;

		$args = $wp_query->query_vars;

		$sql = [
			'join'  => '',
			'where' => '',
		];

		$current_min_price = $current_min_price = null;

		if ( isset( $min_price ) && isset( $max_price ) ) {
			$current_min_price = floatval( $min_price );
			$current_max_price = floatval( $max_price );
		} elseif ( isset( $args['min_price'] ) && isset( $args['max_price'] ) ) {
			// phpcs:enable WordPress.Security.NonceVerification.Recommended
			$current_min_price = floatval( wp_unslash( $args['min_price'] ) );
			$current_max_price = floatval( wp_unslash( $args['max_price'] ) );
		}

		if ( isset( $current_min_price ) && isset( $current_max_price ) ) {
			/**
			 * Adjust if the store taxes are not displayed how they are stored.
			 * Kicks in when prices excluding tax are displayed including tax.
			 */
			if ( wc_tax_enabled() && 'incl' === get_option( 'woocommerce_tax_display_shop' ) && ! wc_prices_include_tax() ) {
				$tax_class = apply_filters( 'woocommerce_price_filter_widget_tax_class', '' ); // Uses standard tax class.
				$tax_rates = \WC_Tax::get_rates( $tax_class );

				if ( $tax_rates ) {
					$current_min_price -= \WC_Tax::get_tax_total( \WC_Tax::calc_inclusive_tax( $current_min_price, $tax_rates ) );
					$current_max_price -= \WC_Tax::get_tax_total( \WC_Tax::calc_inclusive_tax( $current_max_price, $tax_rates ) );
				}
			}

			$sql['join']  = " LEFT JOIN {$wpdb->wc_product_meta_lookup} wc_product_meta_lookup ON $wpdb->posts.ID = wc_product_meta_lookup.product_id ";
			$sql['where'] = $wpdb->prepare(
				' AND NOT (%f<wc_product_meta_lookup.min_price OR %f>wc_product_meta_lookup.max_price ) ',
				$current_max_price,
				$current_min_price
			);
		}

		return $sql;
	}

	public function get_tax_query( $tax_query = array(), $main_query = false ) {
		if ( ! is_array( $tax_query ) ) {
			$tax_query = array(
				'relation' => 'AND',
			);
		}

		// Layered nav filters on terms.
		foreach ( self::get_layered_nav_chosen_attributes() as $taxonomy => $data ) {
			$tax_query[] = array(
				'taxonomy'         => $taxonomy,
				'field'            => 'slug',
				'terms'            => $data['terms'],
				'operator'         => 'and' === $data['query_type'] ? 'AND' : 'IN',
				'include_children' => false,
			);
		}

		$product_visibility_terms  = wc_get_product_visibility_term_ids();
		$product_visibility_not_in = array( is_search() && $main_query ? $product_visibility_terms['exclude-from-search'] : $product_visibility_terms['exclude-from-catalog'] );

		// Hide out of stock products.
		if ( 'yes' === get_option( 'woocommerce_hide_out_of_stock_items' ) ) {
			$product_visibility_not_in[] = $product_visibility_terms['outofstock'];
		}

		// phpcs:disable WordPress.Security.NonceVerification.Recommended
		// Filter by rating.
		if ( isset( $_GET['rating_filter'] ) ) {
			// phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$rating_filter = array_filter( array_map( 'absint', explode( ',', wp_unslash( $_GET['rating_filter'] ) ) ) );
			$rating_terms  = array();
			for ( $i = 1; $i <= 5; $i++ ) {
				if ( in_array( $i, $rating_filter, true ) && isset( $product_visibility_terms[ 'rated-' . $i ] ) ) {
					$rating_terms[] = $product_visibility_terms[ 'rated-' . $i ];
				}
			}
			if ( ! empty( $rating_terms ) ) {
				$tax_query[] = array(
					'taxonomy'      => 'product_visibility',
					'field'         => 'term_taxonomy_id',
					'terms'         => $rating_terms,
					'operator'      => 'IN',
					'rating_filter' => true,
				);
			}
		}
		// phpcs:enable WordPress.Security.NonceVerification.Recommended

		if ( ! empty( $product_visibility_not_in ) ) {
			$tax_query[] = array(
				'taxonomy' => 'product_visibility',
				'field'    => 'term_taxonomy_id',
				'terms'    => $product_visibility_not_in,
				'operator' => 'NOT IN',
			);
		}

		return array_filter( apply_filters( 'minimog/product_query/tax_query', $tax_query, $this ) );
	}

	/**
	 * Get an array of attributes and terms selected with the layered nav widget.
	 *
	 * @return array
	 */
	public static function get_layered_nav_chosen_attributes() {
		// phpcs:disable WordPress.Security.NonceVerification.Recommended
		if ( ! is_array( self::$chosen_attributes ) ) {
			self::$chosen_attributes = array();

			if ( ! empty( $_GET ) ) {
				foreach ( $_GET as $key => $value ) {
					if ( 0 === strpos( $key, 'filter_' ) ) {
						$attribute = wc_sanitize_taxonomy_name( str_replace( 'filter_', '', $key ) );
						$taxonomy  = wc_attribute_taxonomy_name( $attribute );

						$filter_terms = ! empty( $value ) ? explode( ',', wc_clean( wp_unslash( $value ) ) ) : array();

						if ( empty( $filter_terms ) || ! taxonomy_exists( $taxonomy ) || ! wc_attribute_taxonomy_id_by_name( $attribute ) ) {
							continue;
						}

						$query_type                                         = ! empty( $_GET[ 'query_type_' . $attribute ] ) && in_array( $_GET[ 'query_type_' . $attribute ], array(
							'and',
							'or',
						), true ) ? wc_clean( wp_unslash( $_GET[ 'query_type_' . $attribute ] ) ) : '';
						self::$chosen_attributes[ $taxonomy ]['terms']      = array_map( 'sanitize_title', $filter_terms ); // Ensures correct encoding.
						self::$chosen_attributes[ $taxonomy ]['query_type'] = $query_type ? $query_type : apply_filters( 'minimog/layered_nav/default_query_type', 'and' );
					}
				}
			}
		}

		return self::$chosen_attributes;
		// phpcs:disable WordPress.Security.NonceVerification.Recommended
	}

	public static function set_meta_query_min_price( $meta_query, $min_price ) {
		if ( ! is_array( $meta_query ) ) {
			$meta_query = array();
		}

		$meta_query[] = array(
			'key'     => '_price',
			'value'   => $min_price,
			'compare' => '>=',
			'type'    => 'NUMERIC',
		);

		return $meta_query;
	}

	public static function set_meta_query_max_price( $meta_query, $max_price ) {
		if ( ! is_array( $meta_query ) ) {
			$meta_query = array();
		}

		$meta_query[] = array(
			'key'     => '_price',
			'value'   => $max_price,
			'compare' => '<=',
			'type'    => 'NUMERIC',
		);

		return $meta_query;
	}
}

Product_Query::instance()->initialize();
