<?php

namespace Minimog\Woo;

defined( 'ABSPATH' ) || exit;

class Product_Question {
	protected static $instance = null;

	public static function instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	public function initialize() {
		// Support avatars for `question` comment type.
		add_filter( 'get_avatar_comment_types', [ $this, 'add_avatar_for_question_comment_type' ] );

		add_action( 'wp_ajax_nopriv_minimog_add_comment', [ $this, 'add_comment' ] );
		add_action( 'wp_ajax_minimog_add_comment', [ $this, 'add_comment' ] );

		add_action( 'wp_ajax_nopriv_minimog_get_questions', [ $this, 'get_product_question_via_ajax' ] );
		add_action( 'wp_ajax_minimog_get_questions', [ $this, 'get_product_question_via_ajax' ] );
	}

	/**
	 * Make sure WP displays avatars for comments with the `question` type.
	 *
	 * @since  2.3
	 *
	 * @param  array $comment_types Comment types.
	 *
	 * @return array
	 */
	public function add_avatar_for_question_comment_type( $comment_types ) {
		return array_merge( $comment_types, array( 'question' ) );
	}

	public function get_comment_per_page() {
		return apply_filters( 'minimog/product_questions/per_page', get_option( 'comments_per_page' ), $this );
	}

	public function get_current_user() {
		global $user_ID;
		if ( $user_ID ) {
			$user = get_userdata( $user_ID );
		} else {
			$user = wp_set_current_user( 0 );
		}

		return $user;
	}

	public function get_user_current_display_name( $current_user ) {
		$displayName = trim( $current_user->display_name );
		if ( ! $displayName ) {
			$user_nicename = trim( $current_user->user_nicename );
			$displayName   = $user_nicename ? $user_nicename : trim( $current_user->user_login );
		}

		return $displayName;
	}

	public function get_comment_list_args( $args = array() ) {
		$args = wp_parse_args( $args, array(
			'style'       => 'ol',
			'callback'    => array( $this, 'question_template' ),
			'avatar_size' => \Minimog::COMMENT_AVATAR_SIZE,
			'echo'        => false,
		) );

		return $args;
	}

	public function add_comment() {
		if ( ! check_ajax_referer( 'product_question', 'product_question_nonce' ) ) {
			wp_die();
		}

		$post_id            = ! empty( $_POST['post_id'] ) ? absint( $_POST['post_id'] ) : 0;
		$question_parent_id = ! empty( $_POST['question_parent_id'] ) ? absint( $_POST['question_parent_id'] ) : 0;
		$question           = sanitize_textarea_field( $_POST['question'] );

		$current_user = $this->get_current_user();
		if ( $current_user && $current_user->ID ) {
			$user_id = $current_user->ID;
			$name    = $this->get_user_current_display_name( $current_user );
			$email   = $current_user->user_email;
		} else {
			$user_id = 0;
			$name    = sanitize_text_field( $_POST['author'] );
			$email   = sanitize_email( $_POST['email'] );
		}

		if ( empty( $question ) || empty( $name ) || empty( $email ) ) {
			wp_send_json_error( [
				'message' => esc_html__( 'Please fill out required fields.', 'minimog' ),
			] );
		}

		if ( ! is_email( $email ) ) {
			wp_send_json_error( [
				'message' => esc_html__( 'Please fill valid email address.', 'minimog' ),
			] );
		}

		$website_url = ! empty( $_POST['url'] ) ? esc_url_raw( $_POST['url'] ) : '';
		$user_agent  = isset( $_SERVER['HTTP_USER_AGENT'] ) ? $_SERVER['HTTP_USER_AGENT'] : '';

		$data = array(
			'user_id'              => $user_id,
			'comment_post_ID'      => $post_id,
			'comment_author'       => $name,
			'comment_author_email' => $email,
			'comment_content'      => $question,
			'comment_author_url'   => $website_url,
			'comment_agent'        => $user_agent,
			'comment_type'         => 'question',
		);

		// Set type of reply comment ( children comments ) is comment instead of question.
		if ( $question_parent_id ) {
			$parent_comment         = get_comment( $question_parent_id );
			$parent_comment_id      = isset( $parent_comment->comment_ID ) ? $parent_comment->comment_ID : 0;
			$data['comment_parent'] = $parent_comment_id;
			$data['comment_type']   = 'comment';
		}

		$new_comment_id = wp_new_comment( wp_slash( $data ) );
		$new_comment    = get_comment( $new_comment_id );

		$list_comment_args = $this->get_comment_list_args();

		$response = [
			'single_text' => esc_html__( 'Question', 'minimog' ),
			'plural_text' => esc_html__( 'Questions', 'minimog' ),
			'message'     => esc_html__( 'Thanks for asking. We will get back to you as soon as possible.', 'minimog' ),
			'response'    => wp_list_comments( $list_comment_args, array( $new_comment ) ),
		];

		wp_send_json_success( $response );
	}

	public function get_product_question_via_ajax() {
		$keyword      = ! empty( $_GET['keyword'] ) ? sanitize_text_field( $_GET['keyword'] ) : '';
		$current_page = ! empty( $_GET['current_page'] ) ? absint( $_GET['current_page'] ) : 1;
		$post_id      = ! empty( $_GET['post_id'] ) ? absint( $_GET['post_id'] ) : 0;

		$top_questions = get_comments( [
			'type'         => 'question',
			'post_id'      => $post_id,
			'status'       => 'approve',
			'hierarchical' => false,
			// Filter keyword by author, content, email, url..
			'search'       => $keyword,
		] );

		$child_questions = array();

		foreach ( $top_questions as $comment ) {
			$_children_comments = get_comments( array(
				'parent'       => $comment->comment_ID,
				'hierarchical' => true,
				'status'       => 'approve',
			) );

			foreach ( $_children_comments as $_comment ) {
				$child_questions[] = $_comment;
			}
		}

		$all_questions = array_merge( $top_questions, $child_questions );

		$comment_count = count( $top_questions );

		$list_comment_args = $this->get_comment_list_args();

		// Comment Pagination.
		$per_page    = $this->get_comment_per_page();
		$total_pages = ceil( $comment_count / $per_page );

		if ( $current_page > $total_pages ) {
			$current_page = $total_pages;
		}

		$list_comment_args['page']     = $current_page;
		$list_comment_args['per_page'] = $per_page;

		$fragments = [];

		ob_start();
		if ( get_comment_pages_count( $top_questions ) > 1 && get_option( 'page_comments' ) ) {
			?>
			<nav class="navigation question-navigation comment-navigation">
				<h2 class="screen-reader-text"><?php esc_html_e( 'Question navigation', 'minimog' ); ?></h2>

				<div class="comment-nav-links">
					<?php
					\Minimog_Templates::render_paginate_links( [
						'format'  => '?current_page=%#%',
						'current' => max( 1, $current_page ),
						'total'   => $total_pages,
					] );
					?>
				</div>
			</nav>
			<?php
		}
		$fragments['.question-navigation'] = ob_get_clean();

		ob_start();
		wc_get_template( 'single-product/product-question/questions.php', [
			'questions'          => $all_questions,
			'list_comments_args' => $list_comment_args,
		] );
		$fragments['#question-list'] = ob_get_clean();

		wp_send_json_success( [
			'message'   => esc_html__( 'Product question list retrieved successfully.', 'minimog' ),
			'fragments' => $fragments,
		] );
	}

	/**
	 * Question Callback
	 *
	 * @param $comment
	 * @param $args
	 * @param $depth
	 */
	public function question_template( $comment, $args, $depth ) {
		$GLOBALS['comment'] = $comment;

		wc_get_template(
			'single-product/question.php',
			array(
				'comment' => $comment,
				'args'    => $args,
				'depth'   => $depth,
			)
		);
	}

	function question_reply_and_cancel_link( $args, $comment ) {
		$defaults = array(
			'max_depth' => 0,
			'depth'     => 0,
		);

		$args = wp_parse_args( $args, $defaults );

		if ( 0 == $args['depth'] || $args['max_depth'] <= $args['depth'] ) {
			return;
		}

		$enable = apply_filters( 'minimog/enable_question_reply_link', true );
		if ( ! $enable ) {
			return;
		}

		$comment = get_comment( $comment );

		if ( empty( $comment ) ) {
			return;
		}

		if ( empty( $post ) ) {
			$post = $comment->comment_post_ID;
		}

		$post = get_post( $post );

		if ( ! comments_open( $post->ID ) ) {
			return;
		}

		echo '<div class="reply-action">';

		echo sprintf(
			'<a href="%s" class="question-reply question-reply--%s" data-post-id="%s" data-question-id="%s">%s</a>',
			add_query_arg( array(
				'reply_to_question' => $comment->comment_ID,
			), get_permalink( $comment->comment_post_ID ) ),
			esc_attr( $comment->comment_ID ),
			esc_attr( $comment->comment_post_ID ),
			esc_attr( $comment->comment_ID ),
			esc_html__( 'Reply', 'minimog' )
		);

		echo sprintf(
			'<a href="#" class="cancel-reply cancel-reply--%s" data-question-id="%s">%s</a>',
			esc_attr( $comment->comment_ID ),
			esc_attr( $comment->comment_ID ),
			esc_html__( 'Cancel Reply', 'minimog' )
		);

		echo '</div>';
	}

	/**
	 * Question Form
	 */
	public function question_form() {
		wc_get_template( 'product-question-form.php' );
	}
}

Product_Question::instance()->initialize();
